' EEPROM DATA GENERATOR PROGRAM #1 By "Labguy" (R. Diehl) 20201106
'
' This program is for creating the state machine EEPROM data for the "10-4 Eleanor RGBY"
' The NBTV Video Signal Generator project at Labguy's World dot com.
'
' The memory chips are EEPROM (electrically eraseable programmable read only memory) of
' brand and type: ST M29F040B, 4Mb or 512KB capacity. One chip is used for system state
' timing pulses, three other chips are used for storage of 256 frames of red, green and
' blue video. The entire system will fit on a very compact 4 inch by 5 inch hand wired
' prototyping circuit board.
'
' Referring to the State Machine memory chip, each bit represents a different pulse
' stream required by the video format of:
'
' 32 lines by 12.5 frames per second. The entire sync and timing stream is stored in this
' memory chip.
'
' The master clock is 4.096MHz with a single cycle time of 244.14nS. 10,240 clocks make up
' one television line running at 400 lines per second, 32 lines make up one frame of video.
' The counters provide a total of 19 address lines for addressing memory up to 524,288 byte
' locations, of which only 227,680 byte locations are actually used.
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'  THIS PROGRAM PRODUCED THE FIRST ORDER OPERATIONAL TIMING PATTERN ON 20201113
'
'  FRIDAY THE 13TH, OF COURSE. THERE ARE OTHER BUGS IN THE HARDWARE NOT RELATED TO THIS.
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
' - - - - - - -
' PRIMARY SYSTEM TIMING PULSES ARE AS FOLLOWS:
'
' MCLK, Master Clock Frquency          = 4.0960 MHz    (crystal oscillator)
'
' PXL_CLK, Pixel Clock frequency       = 25.600 KHz    (MCLK / 160)
'
' FRAME_RST-NOT, Frame Clock frequency = 12.5 Hz       (MCLK / 327,680)
'
' COMP_SYNC, Composite Sync            = 400 Hz sync, with a missing pulse between TV line 32
'                                        and TV line 1
'
' COMP_BLNK, Composite Blanking        = Line blanking AND frame blanking pulses combined for
'                                        driving frame memory chip output enable off and on.
'                                        DACs automatically put out zero volts, by way of eight
'                                        4.7K pull down resistors on their data bus, when the
'                                        memory chip outputs are forced into the Hi-Z state.
'
' - - - - - - -
' EEPROM BIT DEFINITIONS:
'
' D0 - SMRST-NOT,Active Low    - State machine reset pulse that closes the loop on the state machine cycle.
' D1 - PXL_CLK, Active High    - The pulse that drives the video memory address counters.
' D2 - FRM_RST-NOT, Active Low - Start pulse of a frame scan cycle. Resets the frame address counters.
' D3 - COMP_SYNC, Active High  - Complete sync stream that is added to DAC outputs to make composite video.
' D4 - COMP_BLNK, Active High  - Blanking stream that disables the EEPROM outputs forcing video to black level.
' D5 - Spare bit 1
' D6 - Spare bit 2
' D7 - Spare bit 3
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CLS
PRINT "Sync Prom File Creator for Labguy's 10-4 Eleanor RGBY project"

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE ALL PROGRAM VARIABLES
'
Path$ = "D:\DataFiles\HistoryData\HARDWARE_PROJECTS\_NBTV_TELEVISOR\004_10-4_Eleanor_RGBY\STATE_MACHINE_EEPROM\"
File$ = Path$ + "EleanorStateMachine.bin" 'System State Machine EEPROM file folder and file name

DIM FileDataByte AS _UNSIGNED _BYTE
DIM DataByteMask AS _UNSIGNED _BYTE
DIM FileNumber AS _UNSIGNED LONG
DIM FilePointer AS _UNSIGNED LONG
DIM PulseCounter AS _UNSIGNED LONG
DIM FileOffSet AS _UNSIGNED LONG
DIM LineCounter AS _UNSIGNED LONG

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' CREATE THE FILE, OVERWRITE ANY EXISTING FILE, INITIALIZE THE SYNC MEMORY BINARY FILE,
' FILL ALL 4,194,304 BIT LOCATIONS WITH ONES. THEN INITIALIZE TO LOW EACH BIT POSITION
' THAT WILL CONTAIN ACTIVE HIGH PULSES IN ALL ACTIVE STATE LOCATIONS. YES. WE COULD
' INITIALIZE THE LOWS IN THE FIRST STEP. BUT, THIS IS MORE LOGICAL AND ALLOWS FOR FUTURE
' MODIFICATION OF BIT POSITIONS IF THE NEED ARISES.
'
' Open the file
FileNumber = FREEFILE
OPEN File$ FOR BINARY AS #FileNumber
PRINT "opening ", File$


' Set byte to all binary ones 1111 1111
FileDataByte = 255

PRINT "Filling EEPROM with all ones"

' Count all 524,288 memory chip locations and fill them with ones
FOR FilePointer = 0 TO 524287

    ' Write eight ones into each byte location
    PUT #FileNumber, FilePointer + 1, FileDataByte

    'Do it again until done
NEXT FilePointer

' - - - - - - -
' Prepare each bit position for active hi or low pulses
' The necessary pattern is 11100101 binary = 229 decimal for default init of state machine full range
'
DataByteMask = 229

PRINT DataByteMask

PRINT "Filling state machine range with default 11100101 binary, 229 decimal"

' Count the 327,680 states, plus one, and point to the byte location
FOR FilePointer = 0 TO 327680

    ' Fetch the current byte
    GET #FileNumber, FilePointer + 1, FileDataByte

    ' Insert the low state(s) where needed
    FileDataByte = FileDataByte AND DataByteMask

    ' Write the byte back to the file
    PUT #FileNumber, FilePointer + 1, FileDataByte

    ' Do it again until done
NEXT FilePointer

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D0, SMRST-NOT, SYNC STATE MACHINE RESET, POINT TO LAST STATE LOCATION
' MINUS CLOCKED LATCH DELAYS (TBD, should be 2)
'
'
D0:

PRINT "Writing State Machine Reset bit D0"

' Prepare the bit position 0 for an active low pulse, binary 1111 1110
DataByteMask = 254

'Define the State Machine Reset byte location, -2 delay clks compensation.
FilePointer = 327677

' Point at desired byte location
'SEEK #FileNumber, FilePointer

PRINT FilePointer, DataByteMask

' Read in the desired byte
GET #FileNumber, FilePointer + 1, FileDataByte

' Make D0 low in the byte
FileDataByte = FileDataByte AND DataByteMask

' Write the byte back to the location
PUT #FileNumber, FilePointer + 1, FileDataByte


' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D1, PXL_CLK
'

D1:

PRINT "Initializing Pixel Clock"

' Prepare the bit position 1 for an active high pulse, binary 0000 0010
DataByteMask = 2

' The PXL_CLK will be low for 75 MCLKs and high for 76 MCLKS when active
' Start at 75 MCLKs after the falling edge of CompBlnk, MCLK number 575 + 75 = 650.
StartOffSet = 650

' Count the lines
FOR LineCounter = 0 TO 31
    ' Count the sync pulses per line
    FOR PulseCounter = 0 TO 63
        ' Count the MCLKs per pulse
        FOR HighCounter = 0 TO 75

            ' Define the byte location in the file
            FilePointer = StartOffSet + (10240 * LineCounter) + (151 * PulseCounter) + HighCounter

            ' Read in the desired byte
            GET #FileNumber, FilePointer + 1, FileDataByte

            ' Make D1 high in the byte
            FileDataByte = FileDataByte OR DataByteMask

            ' Write the byte back to the location
            PUT #FileNumber, FilePointer + 1, FileDataByte

        NEXT HighCounter
        ' Do this until done
    NEXT PulseCounter
NEXT LineCounter

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D2, FRAME RESET NOT
'
D2:

PRINT "Initializing Frame Reset Not"

' Prepare the bit position 2 for an active low pulse, binary 1111 1011
DataByteMask = 251

' Start at Clock number 0
FileOffSet = 0

' Count the 48 MCLKs
FOR ClkCounter = 0 TO 47

    ' Define the byte location in the file
    FilePointer = FileOffSet + ClkCounter

    ' Point at desired byte location
    'SEEK #FileNumber, FilePointer + 1

    ' Read in the desired byte
    GET #FileNumber, FilePointer + 1, FileDataByte

    ' Make D2 low in the byte
    FileDataByte = FileDataByte AND DataByteMask

    ' Write the byte back to the location
    PUT #FileNumber, FilePointer + 1, FileDataByte

NEXT ClkCounter

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D3, COMP_SYNC
'
D3:

PRINT "Initializing Composite Sync"

' Prepare the bit position 3 for an active high pulse, binary 0000 1000
DataByteMask = 8

' Start at MCLK number 48 plus one extra line as first sync pulse is missing
StartOffSet = 48 + 10240

' Count the lines (n - 1)
FOR LineCounter = 0 TO 30
    ' Count the MCLKs per pulse
    FOR HighCounter = 0 TO 479

        ' Define the byte location in the file
        FilePointer = StartOffSet + (10240 * LineCounter) + HighCounter

        ' Read in the desired byte
        GET #FileNumber, FilePointer + 1, FileDataByte

        ' Make D1 high in the byte
        FileDataByte = FileDataByte OR DataByteMask

        ' Write the byte back to the location
        PUT #FileNumber, FilePointer + 1, FileDataByte

    NEXT HighCounter
    ' Do this until done
NEXT LineCounter


' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D4, Composite Blanking
'
D4:
PRINT "Initializing Composite Blanking"

' Prepare the bit position 4 for an active high pulse, binary 0001 0000
DataByteMask = 16

' Start at MCLK number 0, this is the key pulse that all others count off of
StartOffSet = 0

' Count the lines
FOR LineCounter = 0 TO 31
    ' Count the MCLKs per pulse
    FOR HighCounter = 0 TO 575

        ' Define the byte location in the file
        FilePointer = StartOffSet + (10240 * LineCounter) + HighCounter

        ' Read in the desired byte
        GET #FileNumber, FilePointer + 1, FileDataByte

        ' Make D1 high in the byte
        FileDataByte = FileDataByte OR DataByteMask

        ' Write the byte back to the location
        PUT #FileNumber, FilePointer + 1, FileDataByte

    NEXT HighCounter
    ' Do this until done
NEXT LineCounter

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D5, not used
'
D5:

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D6, not used
'
D6:

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' INITIALIZE D7, not used
'
D7:

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' ALL DONE, CLOSE THE SYNC MEMORY BINARY FILE
'

CLOSE #FileNumber

PRINT "All done!"

END


